(function(){
	"use strict";
	var QueryFilter = module.exports = function(opts){
		var queryFilter = this;
		Object.keys(opts || {}).forEach(function(key){
			queryFilter[key] = opts[key];
		});
	};
	QueryFilter.prototype.inverse = false;
	QueryFilter.prototype.exact = false;
	QueryFilter.prototype.name = null;
	QueryFilter.prototype.key = null;
	QueryFilter.prototype.type = null;
	QueryFilter.prototype.names = {
		inverse: 'does not',
		exact: 'exactly',
		noOperator: 'match',
		gtOperator: 'greater than',
		ltOperator: 'less than',
		btOperator: 'between'
	};
	QueryFilter.prototype.operator = null;
	QueryFilter.prototype.value = null;
	QueryFilter.prototype.toString = function(){
		var queryParts = [];
		queryParts.push(this.key);
		if ( this.inverse === true )   queryParts.push('!');
		if ( this.exact === true )     queryParts.push('=');
		if ( this.operator )           queryParts.push(this.operator);
		queryParts.push(this.value);
		return queryParts.join(':');
	};
	QueryFilter.prototype.fromString = function(queryString){
		var queryFilter = this;
		var queryParts = queryString.split(':');
		this.value = queryParts.pop();
		this.inverse = false;
		this.exact = false;
		queryParts.forEach(function(value){
			if ( value === '!' ) {
				queryFilter.inverse = true;
			}
			else if ( value === '=' ) {
				queryFilter.exact = true;
			}
			else {
				queryFilter.operator = value;
			}
		});
	};
	QueryFilter.prototype.toHumanString = function(){
		var converseParts = [];
		converseParts.push(this.name || this.key);
		if ( this.inverse === true ) {
			converseParts.push(this.names.inverse);
		}
		if ( this.exact === true ) {
			converseParts.push(this.names.exact);
		}
		var operator = this.operator || 'no';
		converseParts.push(this.names[operator+'Operator']);
		var value = this.value;
		if ( Array.isArray(value) ) {
			if ( this.type === 'location' ) {
				value = value.join(', ');
			} else {
				value = value.join(' and ');
			}
		} else {
			value = value.toString();
		}
		converseParts.push(value);
		return converseParts.join(' ');
	};
})()